@@ -23,10 +23,27 @@ import classeval.ROC as ROC
 
 # %% Main function for all two class results.
 def plot(out, title='', fontsize=12, figsize=(20,15)):
+    """Make plot based on evaluated model.
 
+    Parameters
+    ----------
+    out : dict
+        Evaluated model from the eval() function.
+    title : str, optional
+        Title of the figure. The default is ''.
+    fontsize : int, optional
+        Font-size. The default is 12.
+    figsize : tuple, optional
+        Figure size. The default is (20,15).
+
+    Returns
+    -------
+    ax : Object
+
+    """
     y_true = out['y_true']
     y_proba = out['y_proba']
-    threshold = out['threshold']
+    threshold = out.get('threshold', None)
 
     if len(out['class_names'])==2:
         # Setup figure
@@ -43,6 +60,11 @@ def plot(out, title='', fontsize=12, figsize=(20,15)):
         plt.show()
         # Confusion matrix
         _ = confmatrix.plot(out['confmat'], title=title, cmap=plt.cm.Blues)
+        # Stacked bar
+        class_names = {True:str(out['pos_label']), False:str(out['neg_label'])}
+        y_true_str = np.array(list(map(lambda x: class_names.get(x), y_true)))
+        y_pred_str = np.array(list(map(lambda x: class_names.get(x), out['y_pred'])))
+        _ = _stackedbar_multiclass(y_true_str, y_pred_str, showfig=True, fontsize=fontsize)
     elif len(out['class_names'])>2:
         # Setup figure
         # [fig, ax] = plt.subplots(2,1,figsize=figsize)
@@ -55,7 +77,7 @@ def plot(out, title='', fontsize=12, figsize=(20,15)):
 
 
 # %% Main function for all two class results.
-def eval(y_true, y_proba, y_score=None, y_pred=None, pos_label=None, threshold=0.5, verbose=3):
+def eval(y_true, y_proba, y_score=None, y_pred=None, pos_label=None, threshold=0.5, normalize=False, verbose=3):
     """Evaluate and make plots for two-class models.
 
     Parameters
@@ -64,12 +86,16 @@ def eval(y_true, y_proba, y_score=None, y_pred=None, pos_label=None, threshold=0
         True labels of the classes.
     y_proba : array of floats
         Probabilities of the predicted labels.
+    y_score : array of floats
+        decision_function for the predicted labels. (only required in case of multi-class)
+    y_pred : array-like
+        Predicted labels from model.
+    pos_label : str
+        Positive label (only for the two-class models and when y_true is of type string. If you set bool, then the positive label is True)
     threshold : float [0-1], optional
         Cut-off point to define the class label. The default is 0.5 in a two-class model.
-    title : str, optional
-        Title of the plot. The default is ''.
-    showfig : bool, optional
-        Show the figure to screen. The default is True.
+    normalize : bool, optional
+        Normalize the values in the confusion matrix. The default is False.
     verbose : int, optional
         print message to screen. The default is 3.
 
@@ -89,9 +115,9 @@ def eval(y_true, y_proba, y_score=None, y_pred=None, pos_label=None, threshold=0
     # Create classification report
     class_names = np.unique(y_true)
     if len(class_names)==2:
-        out = eval_twoclass(y_true, y_proba, threshold=threshold, pos_label=pos_label, verbose=verbose)
+        out = eval_twoclass(y_true, y_proba, threshold=threshold, pos_label=pos_label, normalize=normalize, verbose=verbose)
     elif len(class_names)>2:
-        out = eval_multiclass(y_true, y_proba, y_score, y_pred, threshold=threshold, verbose=verbose)
+        out = eval_multiclass(y_true, y_proba, y_score, y_pred, normalize=normalize, verbose=verbose)
     else:
         out=None
 
@@ -100,25 +126,7 @@ def eval(y_true, y_proba, y_score=None, y_pred=None, pos_label=None, threshold=0
 
 
 # %% Two class results
-def _stackedbar_multiclass(y_true, y_pred, fontsize=12, showfig=False, verbose=3):
-    uiy = np.unique(y_true)
-    df = pd.DataFrame(data=np.zeros((3,3)), index=uiy, columns=uiy)
-    for i, y in enumerate(uiy):
-        I = y_true==y
-        labels, n = np.unique(y_pred[I], return_counts=True)
-        df[y].loc[labels]=n
-
-    if showfig:
-        df.plot(kind='bar', stacked=True)
-        plt.ylabel('Number of predicted classes', fontsize=fontsize)
-        plt.xlabel('True class', fontsize=12)
-        plt.title('Class prediction', fontsize=12)
-
-    return(df)
-
-
-# %% Two class results
-def eval_multiclass(y_true, y_proba, y_score, y_pred, threshold=0.5, normalize=False, verbose=3):
+def eval_multiclass(y_true, y_proba, y_score, y_pred, normalize=False, verbose=3):
     """Evaluate for multi-class model.
 
     Parameters
@@ -131,8 +139,6 @@ def eval_multiclass(y_true, y_proba, y_score, y_pred, threshold=0.5, normalize=F
         decision_function for the predicted labels. (only required in case of multi-class)
     y_pred : array-like
         Predicted labels from model.
-    threshold : float [0-1], optional
-        Cut-off point to define the class label. The default is 0.5 in a two-class model.
     normalize : bool, optional
         Normalize the values in the confusion matrix. The default is False.
     verbose : int, optional
@@ -151,7 +157,6 @@ def eval_multiclass(y_true, y_proba, y_score, y_pred, threshold=0.5, normalize=F
     out['y_true'] = y_true
     out['y_pred'] = y_pred
     out['y_proba'] = y_proba
-    out['threshold'] = threshold
     out['class_names'] = np.unique(y_true)
     out['ROCAUC'] = roc_scores
     out['stackbar'] = stackbar
@@ -184,18 +189,17 @@ def eval_twoclass(y_true, y_proba, pos_label=None, threshold=0.5, normalize=Fals
     dict containing results.
 
     """
-    class_names = np.unique(y_true)
     y_label = y_true.astype(str)
     y_pred = y_proba>=threshold
 
-    if len(class_names)>2:
+    if len(np.unique(y_true))>2:
         raise Exception('[classeval] This function is to evaluate two-class models and not multi-class.')
     if (pos_label is None) and (y_true.dtype!='bool'):
         raise Exception('[classeval] CAP should have input argument <pos_label> or <y_true> being of type bool.')
     if (pos_label is not None) and (y_true.dtype!='bool'):
         # If y_true is strings, convert to bool based on positive label
         y_true = y_label==pos_label
-        neg_label = np.setdiff1d(class_names,pos_label)[0]
+        neg_label = np.setdiff1d(np.unique(y_label),pos_label)[0]
         # Determine y_pred
         y_pred_label = np.repeat('',len(y_true)).astype(y_label.dtype)
         I = y_proba>=threshold
@@ -206,6 +210,11 @@ def eval_twoclass(y_true, y_proba, pos_label=None, threshold=0.5, normalize=Fals
         neg_label=False
         y_pred_label = y_pred.astype(str)
 
+    # In sklearn, the output of the proba function has 2 columns.
+    # The first column is the probability that the entry has the -1 label
+    # The second column is the probability that the entry has the +1 label
+    class_names = [neg_label, pos_label]
+
     # ROC plot
     ROC_scores = ROC.eval(y_true, y_proba, threshold=threshold, verbose=verbose)
     if verbose>=3: print('[classeval] AUC: %.2f' %(ROC_scores['auc']))
@@ -575,3 +584,21 @@ def load_example(data='breast'):
         y[y=='0']='dead'
         X.index = y
     return X, y
+
+# %% Two class results
+def _stackedbar_multiclass(y_true, y_pred, fontsize=12, showfig=False, verbose=3):
+    uiy = np.unique(y_true)
+    df = pd.DataFrame(data=np.zeros((len(uiy),len(uiy))), index=uiy, columns=uiy)
+    for i, y in enumerate(uiy):
+        I = y_true==y
+        labels, n = np.unique(y_pred[I], return_counts=True)
+        df[y].loc[labels]=n
+
+    if showfig:
+        df.plot(kind='bar', stacked=True)
+        plt.ylabel('Number of predicted classes', fontsize=fontsize)
+        plt.xlabel('True class', fontsize=12)
+        plt.title('Class prediction', fontsize=12)
+        plt.grid(True)
+
+    return(df)
